#include <stdio.h> 
#include <ctype.h> 
#include <sys/types.h> 
#include <string.h> 
#include <pwd.h> 
 
#if defined(SUNOS4) || defined(DUNIX) 
#include <ufs/quota.h> 
#if defined(SUNOS4) 
#include <mntent.h> 
#endif 
#elif defined(SUNOS5) 
#include <unistd.h> 
#include <errno.h> 
#include <sys/fs/ufs_quota.h> 
#endif 
 
#include <rpc/rpc.h> 
#include "rquota.h" 
 
extern long strtol(); 
 
static int show_local_quota(char *fs, uid_t uid); 
static int getquota(char *mntpoint, uid_t uid, struct dqblk *dqbp); 
static int show_nfs_quota(char *host, char *fs, uid_t uid); 
static void showquota(getquota_rslt *resultp); 
 
getquota_rslt *rquotaproc_getquota_1(); 
getquota_rslt *rquotaproc_getactivequota_1(); 
 
int 
main(argc, argv) 
	int argc; 
	char **argv; 
{ 
	char *host; 
	char *fs; 
	uid_t uid; 
	char *p; 
	struct passwd *pw; 
 
	if (argc != 3) { 
	 (void) fprintf(stderr, "Usage: quotatest <fs> <user>\n"); 
	 return 1; 
	} 
 
	host = argv[1]; 
	fs = strchr(argv[1], ':'); 
	if (fs == NULL) { 
	 /* 
	 * No "host:"; it's a local file system. 
	 */ 
	 host = NULL; 
	 fs = argv[1]; 
	} else 
	 *fs++ = '\0'; 
 
	if (isdigit((unsigned char)(*argv[2]))) { 
	 uid = strtol(argv[2], &p, 10); 
	 if (p == argv[2] || *p != '\0') { 
	 (void) fprintf(stderr, 
	 "quotatest: %s is not a valid user ID number\n", 
	 argv[2]); 
	 return 1; 
	 } 
	} else { 
	 pw = getpwnam(argv[2]); 
	 if (pw == NULL) { 
	 (void) fprintf(stderr, 
	 "quotatest: No such user %s\n", argv[2]); 
	 return 2; 
	 } 
	 uid = pw->pw_uid; 
	} 
 
	if (host == NULL) 
	 return show_local_quota(fs, uid); 
	else 
	 return show_nfs_quota(host, fs, uid); 
} 
 
static int 
show_local_quota(char *mntpoint, uid_t uid) 
{ 
	struct dqblk quotablk; 
 
	if (getquota(mntpoint, uid, &quotablk) != 0) 
	 return 2; 
	(void) printf("bhardlimit = %lu\n", 
	 (unsigned long)quotablk.dqb_bhardlimit); 
	(void) printf("bsoftlimit = %lu\n", 
	 (unsigned long)quotablk.dqb_bsoftlimit); 
	(void) printf("curblocks = %lu\n", 
	 (unsigned long)quotablk.dqb_curblocks); 
#if defined(SUNOS4) || defined(SUNOS5) 
	(void) printf("fhardlimit = %lu\n", 
	 (unsigned long)quotablk.dqb_fhardlimit); 
	(void) printf("fsoftlimit = %lu\n", 
	 (unsigned long)quotablk.dqb_fsoftlimit); 
	(void) printf("curfiles = %lu\n", 
	 (unsigned long)quotablk.dqb_curfiles); 
	(void) printf("btimeleft = %lu\n", 
	 (unsigned long)quotablk.dqb_btimelimit); 
	(void) printf("ftimeleft = %lu\n", 
	 (unsigned long)quotablk.dqb_ftimelimit); 
#elif defined(DUNIX) 
	/* 
	 * Thanks for changing the names, Digital. I really appreciate 
	 * it.... 
	 */ 
	(void) printf("ihardlimit = %lu\n", 
	 (unsigned long)quotablk.dqb_ihardlimit); 
	(void) printf("isoftlimit = %lu\n", 
	 (unsigned long)quotablk.dqb_isoftlimit); 
	(void) printf("curinodes = %lu\n", 
	 (unsigned long)quotablk.dqb_curinodes); 
	(void) printf("btime = %lu\n", 
	 (unsigned long)quotablk.dqb_btime); 
	(void) printf("itime = %lu\n", 
	 (unsigned long)quotablk.dqb_itime); 
#endif 
	return 0; 
} 
 
static int 
getquota(char *mntpoint, uid_t uid, struct dqblk *dqbp) 
{ 
#if defined(SUNOS4) 
	FILE *mtab; 
	struct mntent *mnt; 
 
	mtab = setmntent(MOUNTED, "r"); 
	if (mtab == NULL) { 
	 perror("quotatest: Can't open /etc/mtab"); 
	 return -1; 
	} 
	while ((mnt = getmntent(mtab)) != NULL) { 
	 if (strcmp(mnt->mnt_type, MNTTYPE_42) != 0) 
	 continue;	/* not BSD */ 
	 if (strcmp(mntpoint, mnt->mnt_dir) != 0) 
	 continue;	/* not us */ 
	 goto found; 
	} 
	(void) fprintf(stderr, "quotatest: No file system %s\n", mntpoint); 
	(void) endmntent(mtab); 
	return -1; 
	
found: 
	(void) endmntent(mtab); 
	if (quotactl(Q_GETQUOTA, mnt->mnt_fsname, uid, (caddr_t)dqbp) < 0) { 
	 perror("quotatest: Can't get quota"); 
	 return -1; 
	} 
	return 0; 
#elif defined(DUNIX) 
	if (quotactl(mntpoint, QCMD(Q_GETQUOTA, USRQUOTA), uid, 
	 (caddr_t)dqbp) < 0) { 
	 perror("quotatest: Can't get quota"); 
	 return -1; 
	} 
	return 0; 
#elif defined(SUNOS5) 
	int fd; 
	char quotafile[MAXPATHLEN+1]; 
	struct quotctl quotctl; 
 
	if (strlen(mntpoint) + sizeof "/quotas" > MAXPATHLEN+1) { 
	 fprintf(stderr, "quotatest: Mount point name %s too long\n", 
	 mntpoint); 
	 return -1; 
	} 
	strcpy(quotafile, mntpoint); 
	strcat(quotafile, "/quotas"); 
	fd = open(quotafile, O_RDONLY); 
	if (fd < 0) { 
	 fprintf(stderr, "quotatest: Can't open quota file %s: %s\n", 
	 quotafile, strerror(errno)); 
	 return -1; 
	} 
	quotctl.op = Q_GETQUOTA; 
	quotctl.uid = uid; 
	quotctl.addr = (caddr_t)dqbp; 
	if (ioctl(fd, Q_QUOTACTL, (caddr_t)&quotctl) < 0) { 
	 perror("quotatest: Can't get quota"); 
	 return -1; 
	} 
	return 0; 
#endif 
} 
 
static int 
show_nfs_quota(char *host, char *fs, uid_t uid) 
{ 
	CLIENT *clnt; 
	getquota_args args; 
	getquota_rslt *resultp; 
 
	clnt = clnt_create(host, RQUOTAPROG, RQUOTAVERS, "udp"); 
	if (clnt == NULL) { 
	 (void) fprintf(stderr, 
	 "quotatest: Can't get at rquota server on %s: %s\n", 
	 host, clnt_spcreateerror("")); 
	 return 2; 
	} 
 
	auth_destroy(clnt->cl_auth); 
	clnt->cl_auth = authunix_create_default(); 
 
	args.gqa_pathp = fs; 
	args.gqa_uid = uid; 
	resultp = rquotaproc_getquota_1(&args, clnt); 
	if (resultp == NULL) { 
	 clnt_perror(clnt, "Can't make getquota call to rquota on netapp"); 
	 return 2; 
	} 
	(void) printf("Result of getquota call:\n"); 
	showquota(resultp); 
 
	args.gqa_pathp = fs; 
	args.gqa_uid = uid; 
	resultp = rquotaproc_getactivequota_1(&args, clnt); 
	if (resultp == NULL) { 
	 clnt_perror(clnt, "Can't make getactivequota call to rquota on netapp"); 
	 return 2; 
	} 
	(void) printf("Result of getactivequota call:\n"); 
	showquota(resultp); 
	return 0; 
} 
	
static void 
showquota(getquota_rslt *resultp) 
{ 
	switch (resultp->status) { 
 
	case Q_OK: 
	 (void) printf("bsize = %d\n", 
	 resultp->getquota_rslt_u.gqr_rquota.rq_bsize); 
	 (void) printf("active = %s\n", 
	 resultp->getquota_rslt_u.gqr_rquota.rq_active ? 
	 "yes" : "no"); 
	 (void) printf("bhardlimit = %u\n", 
	 resultp->getquota_rslt_u.gqr_rquota.rq_bhardlimit); 
	 (void) printf("bsoftlimit = %u\n", 
	 resultp->getquota_rslt_u.gqr_rquota.rq_bsoftlimit); 
	 (void) printf("curblocks = %u\n", 
	 resultp->getquota_rslt_u.gqr_rquota.rq_curblocks); 
	 (void) printf("fhardlimit = %u\n", 
	 resultp->getquota_rslt_u.gqr_rquota.rq_fhardlimit); 
	 (void) printf("fsoftlimit = %u\n", 
	 resultp->getquota_rslt_u.gqr_rquota.rq_fsoftlimit); 
	 (void) printf("curfiles = %u\n", 
	 resultp->getquota_rslt_u.gqr_rquota.rq_curfiles); 
	 (void) printf("btimeleft = %u\n", 
	 resultp->getquota_rslt_u.gqr_rquota.rq_btimeleft); 
	 (void) printf("ftimeleft = %u\n", 
	 resultp->getquota_rslt_u.gqr_rquota.rq_ftimeleft); 
	 break; 
 
	case Q_NOQUOTA: 
	 (void) printf("No quota for you\n"); 
	 break; 
 
	case Q_EPERM: 
	 (void) printf("No permission\n"); 
	 break; 
	} 
} 
