#ifndef _RPC_QUOTA_H
#define _RPC_QUOTA_H

#include <string.h> 
#include <unistd.h> 
#include <rpc/rpc.h> 
#include <sys/param.h>
#include <sys/types.h> 
 
#include "rpc-quota/rquota_xdr.c"
#include "rpc-quota/rquota_clnt.c"
#include "rquota.h" 
 
int fetchnfsquotas(char *path, uint64_t *quotaused, uint64_t *softquota, uint64_t *hardquota)
{
	char host[MAXPATHLEN+1], *fs;
	uid_t uid;
	CLIENT *clnt; 
	getquota_args args; 
	getquota_rslt *resultp; 
	int blocksize;

	*quotaused = *softquota = *hardquota = 0;

	/* Copy the path so we can insert '\0' to split it into components */
	
	strncpy(host, path, sizeof(host)-1); 
	host[sizeof(host)-1] = '\0';

	/* Split the path into host and filestore components, set up the uid. */
	
	fs = strchr(host, ':'); 
	if (fs == NULL)
		return 1;	/* No "host:"; it's a local file system. */ 
	*fs++ = '\0'; 

	uid = getuid();

	/* Create and perform the RPC request. */

	clnt = clnt_create(host, RQUOTAPROG, RQUOTAVERS, "udp"); 
	if (clnt == NULL) 
		return 2; 	/* Can't get at rquota server. */
 
	auth_destroy(clnt->cl_auth); 
	clnt->cl_auth = authunix_create_default(); 
 
	args.gqa_pathp = fs; 
	args.gqa_uid = uid; 
	resultp = rquotaproc_getquota_1(&args, clnt); 
	if (resultp == NULL) 
		return 2;	/* Can't make getquota call to rquota on server. */

	blocksize = resultp->getquota_rslt_u.gqr_rquota.rq_bsize;
 
	switch (resultp->status) { 
		case Q_OK: 
			*quotaused = resultp->getquota_rslt_u.gqr_rquota.rq_curblocks * blocksize / 1024;
			*softquota = resultp->getquota_rslt_u.gqr_rquota.rq_bsoftlimit * blocksize / 1024; 
			*hardquota = resultp->getquota_rslt_u.gqr_rquota.rq_bhardlimit * blocksize / 1024;
			break;

		case Q_NOQUOTA: 
			return 3;	/* No quota for user. */
			break; 
 
		case Q_EPERM: 
			return 4;	/* No permission. */
			break; 
	} 

	return 0; 
}

#endif /* !_RPC_QUOTA_H */
